<?php

namespace Drupal\eca_drush\Plugin\Action;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\eca\Plugin\Action\ConfigurableActionBase;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\Process;

/**
 * Describes the eca_drush execute_drush_command action.
 *
 * @Action(
 *   id = "eca_drush_simple_execute_drush_command",
 *   label = @Translation("Execute Drush command"),
 *   description = @Translation("Runs a specific Drush command with the given
 *   arguments"), eca_version_introduced = "2.1.4" * )
 */
class ExecuteDrushCommandAction extends ConfigurableActionBase {

  protected string $drushExecutablePath = '';

  /**
   * {@inheritdoc}
   */
  public function access($object, ?AccountInterface $account = NULL, $return_as_object = FALSE): bool|AccessResultInterface {
    $access_result = AccessResult::allowed();
    return $return_as_object ? $access_result : $access_result->isAllowed();
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    // @todo: Make this configurable, but is has to match most of the projects.
    $this->drushExecutablePath = '../vendor/bin/drush';
    // Command to execute.
    $command = [$this->drushExecutablePath, $this->configuration['command']];

    $environment = [];
    if (!empty($this->configuration['arguments'])) {
      $arguments = $this->configuration['arguments'];
      // Use preg_split to split by spaces not enclosed within quotes.
      $arguments = preg_split('/\s+(?=(?:[^\'"]|\'[^\']*\'|"[^"]*")*$)/', $arguments);
      foreach ($arguments as $key => $argument) {
        if (str_starts_with($argument, '--uri=')) {
          $environment['DRUSH_OPTIONS_URI'] = str_replace('--uri=', '', $argument);
        }
      }
      $command = array_merge($command, $arguments);
    }

    // Initialize the Symfony Process.
    $process = new Process($command, NULL, ['ECA_DRUSH_COMMAND_CONTEXT' => 'running'] + $environment);

    try {
      $process->mustRun();
      $this->logger->info('Drush command executed successfully: @output', [
        '@output' => $process->getOutput(),
      ]);
    } catch (ProcessFailedException $e) {
      $this->logger->error('Drush command failed: @error', [
        '@error' => $e->getMessage(),
      ]);
    }

  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
        'command' => '',
        'arguments' => '',
      ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    // @todo: Make this configurable, but is has to match most of the projects.
    $this->drushExecutablePath = '../vendor/bin/drush';
    // Command to execute. Replace 'cache-rebuild' with your desired command.
    $command = [$this->drushExecutablePath, 'list', '--format=json'];

    // Initialize the Symfony Process.
    $process = new Process($command);
    $process->mustRun();

    $info = Json::decode($process->getOutput());

    $options = [];

    // Group commands by namespace.
    foreach ($info['commands'] as $command) {
      // Skip hidden commands.
      if (!empty($command['hidden'])) {
        continue;
      }


      $namespace = (string) $this->t('Common');
      // Determine the namespace for grouping; default to 'Common'.
      foreach ($info['namespaces'] as $namespace) {
        if (in_array($command['name'], $namespace['commands'], TRUE)) {
          $namespace = (string) $this->t('@namespace (group)', ['@namespace' => $namespace['id']]);
          break;
        }
      }

      // Add the command to the appropriate namespace group.
      $options[$namespace][$command['name']] = $this->t('@name (@description)', [
        '@name' => $command['name'],
        '@description' => $command['description'],
      ]);
    }

    // Remove empty namespaces/groups.
    $options = array_filter($options, 'count');

    $flat_options = [];
    // Flatten grouping.
    foreach ($options as $namespace => $commands) {
      $flat_options['__namespace__' . $namespace] = $namespace;
      $flat_options += $commands;
    }

    $form['command'] = [
      '#type' => 'select',
      '#required' => TRUE,
      '#title' => $this->t('Drush command'),
      '#options' => ['' => $this->t('- Select -')] + $flat_options,
      '#default_value' => $this->configuration['command'],
      '#description' => $this->t('Application: @name (@version)',
        [
          '@name' => $info['application']['name'],
          '@version' => $info['application']['version'],
        ]
      ),
    ];
    foreach (array_keys($options) as $namespace) {
      $form['command']['__namespace__' . $namespace]['#disabled'] = TRUE;
      $form['command']['__namespace__' . $namespace]['#attributes']['disabled'] = '';
    }


    // Add initially configured arguments.
    $form['arguments'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Command Arguments'),
      '#default_value' => $this->configuration['arguments'],
    ];


    return parent::buildConfigurationForm($form, $form_state);
  }

  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    if (
      ($form_state->getValue('command', '') === '') ||
      str_starts_with($form_state->getValue('command', ''), '__namespace__')
    ) {
      $form_state->setErrorByName('command', $this->t('Please select a command.'));
    }
    parent::validateConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['command'] = $form_state->getValue('command');
    $this->configuration['arguments'] = $form_state->getValue('arguments');
    parent::submitConfigurationForm($form, $form_state);
  }

}
